<?php

namespace App\Http\Controllers\Admin;

use Illuminate\Http\Request;

use App\Http\Requests;
use App\Http\Controllers\Controller;

use \Carbon\Carbon;
use App\Post;
use App\Tag;
use App\Picture;
use Auth;
use Input;
use Form;

class AdminController extends Controller
{
    public function __construct(){
        $this->middleware('auth');
        Form::macro('custom_date', function($val = false, $name){
            $timestamp = strtotime($val);
            $default = $val ? date('Y-m-d', $timestamp) : Carbon::now()->toDateString();
            return '<input id="'.$name.'" class="form-control" type="date" name="'.$name.'" value="'.$default.'">';
        });
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $posts = Post::with('tags')
            ->orderBy('created_at', 'desc')
            ->paginate(10);
        return view('admin.index', compact('posts'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $tags = Tag::all();
        return view('admin.create', compact('tags'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Requests\AdminFormRequest $request)
    {
        $rq = $request->all();
        // Create image if needed
        if(isset($rq['image']))
            $rq = $this->createImage($rq);
        // Creation de l'extrait
        if(isset($rq['contenu']))
            $rq['extrait'] = str_limit($rq['contenu']);
        $post = Post::create($rq);
        // Tags relationship
        if(isset($rq['tags'])){
            foreach($rq['tags'] as $tag){
                $post->tags()->attach($tag);
            }
        }
        return redirect()->to('admin/')->with('message', 'Post créé');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $post = Post::where('id', $id)->with('picture', 'tags')->get();
        $post = $post[0];
        $tags = Tag::all();
        $status = [
            'published' => $post->statut == 'publié',
            'unpublished' => $post->statut == 'dépublié',
            'trash' => $post->statut == 'corbeille'
        ];
        return view('admin.edit', compact('post', 'status', 'tags'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Requests\AdminFormRequest $request, $id)
    {
        $post = Post::find($id);
        $rq = $request->all();
        // Delete image if needed
        if(isset($rq['rm_image']) && $rq['rm_image'] === 'on')
            $rq['picture_id'] = null;
        // Create image if needed
        if(isset($rq['image']))
            $rq = $this->createImage($rq);
        // Refresh tags
        if(isset($rq['tags']))
            $post->tags()->sync($rq['tags']);
        else
            $post->tags()->detach();
        // Creation de l'extrait
        if(isset($rq['contenu']))
            $rq['extrait'] = str_limit($rq['contenu']);
        // Update post
        $post->update($rq);
        return redirect()->to('admin/')->with('message', 'Post '.$id.' mis à jour');
    }

    /**
     * Create an image
     */

    private function createImage($rq){
        $file = $rq['image'];
        $ext = $file->getClientOriginalExtension();
        $size = $file->getSize();
        $filename = str_slug(str_random(rand(20, 30))).'.'.$ext;
        $file->move(env('IMG_PATH'), $filename);
        $image = Picture::create([
            'filename' => $filename,
            'type' => $ext,
            'size' => $size
        ]);
        $rq['picture_id'] = $image->id;
        return $rq;
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        Post::find($id)->delete();
        return redirect()->to('admin/')->with('message', 'Post '.$id.' supprimé');
    }
}
